/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.AbstractWsdlModelItem;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepProperty;

/**
 * Base class for WSDL TestCase test steps.
 *  
 * @author Ole.Matzura
 */

abstract public class WsdlTestStep extends AbstractWsdlModelItem<TestStepConfig> implements TestStep
{
	private final WsdlTestCase testCase;
	private Map<String,TestStepProperty> properties;
	private Set<WsdlTestStepListener> listeners = new HashSet<WsdlTestStepListener>();
	private final boolean forLoadTest;
	private final boolean hasEditor;
   
   protected WsdlTestStep( WsdlTestCase testCase, TestStepConfig config, boolean hasEditor, boolean forLoadTest )
   {
   	super( config, testCase, null );
   	
      this.testCase = testCase;
		this.hasEditor = hasEditor;
		this.forLoadTest = forLoadTest;
   }
   
   public boolean hasEditor()
	{
		return hasEditor;
	}

	public boolean isForLoadTest()
   {
   	return forLoadTest;
   }
   
   /**
    * Called after creation of all teststeps, should be used for inter-test-step initializations
    * @param config 
    */
   
   public void postInit(TestStepConfig config)
   {}

	protected PanelBuilder createPanelBuilder()
	{
		return null;
	}

	public WsdlTestCase getTestCase()
   {
      return testCase;
   }
   
   /**
    * Called from WsdlTestCase when moving a teststep due to no move functionality
    * in xmlbeans generated arrays. 
    * 
    * @param config the new config to use, will be a copy of the existing one. The current
    * will be invalid
    */
   
   public void resetConfigOnMove( TestStepConfig config )
   {
   	setConfig( config );
   }

	public boolean cancel()
	{
		return false;
	}
	
   public String [] getPropertyNames()
	{
   	if( properties == null )
   		return new String[0];
   	
   	String [] result = new String[properties.size()];
   	int ix = 0;
   	for( TestStepProperty property : properties.values() )
   		result[ix++] = property.getName();
   	
   	return result;
	}
   
	public TestStepProperty getProperty(String name)
	{
		return properties == null || name == null ? null : properties.get( name.toUpperCase() );
	}
	
	public String getPropertyValue(String name)
	{
		if( properties == null )
			return null;
		
		TestStepProperty testStepProperty = properties.get( name.toUpperCase());
		return testStepProperty == null ? null : testStepProperty.getValue();
	}

	public void setPropertyValue(String name, String value)
	{
		if( properties == null )
			return;
		
		TestStepProperty testStepProperty = properties.get( name.toUpperCase());
		if( testStepProperty != null )
		{
			testStepProperty.setValue( value );
		}
	}

	protected void addProperty( TestStepProperty property )
	{
		if( properties == null )
			properties = new HashMap<String,TestStepProperty>();
			
		properties.put( property.getName().toUpperCase(), property );
	}
	
	protected void deleteProperty( String name )
	{
		if( properties != null )
			properties.remove( name.toUpperCase() );
	}

	protected void propertyRenamed( String oldName )
	{
		if( properties == null )
			return;
		
		TestStepProperty testStepProperty = properties.get( oldName.toUpperCase() );
		if( testStepProperty == null )
			return;
		
		properties.remove( oldName.toUpperCase() );
		String newName = testStepProperty.getName();
		properties.put( newName.toUpperCase(), testStepProperty );
		
		firePropertyRenamed( oldName, newName );
	}
	
	public void addTestStepListener( WsdlTestStepListener listener )
	{
		listeners.add( listener );
	}
	
	public void removeTestStepListener( WsdlTestStepListener listener )
	{
		listeners.remove( listener );
	}
	
	protected void firePropertyAdded( String name )
	{
		WsdlTestStepListener[] array = listeners.toArray( new WsdlTestStepListener[listeners.size()]);
		for( WsdlTestStepListener listener : array )
		{
			listener.propertyAdded( name );
		}
	}
	
	protected void firePropertyRemoved( String name )
	{
		WsdlTestStepListener[] array = listeners.toArray( new WsdlTestStepListener[listeners.size()]);
		for( WsdlTestStepListener listener : array )
		{
			listener.propertyRemoved( name );
		}
	}
	
	protected void firePropertyRenamed( String oldName, String newName )
	{
		WsdlTestStepListener[] array = listeners.toArray( new WsdlTestStepListener[listeners.size()]);
		for( WsdlTestStepListener listener : array )
		{
			listener.propertyRenamed( oldName, newName );
		}
	}	
	
	protected void firePropertyValueChanged( String name, String oldValue, String newValue )
	{
		if( oldValue == null && newValue == null )
			return;
		
		if( oldValue != null && oldValue.equals( newValue ))
			return;

		if( newValue != null && newValue.equals( oldValue ))
			return;
		
		WsdlTestStepListener[] array = listeners.toArray( new WsdlTestStepListener[listeners.size()]);
		for( WsdlTestStepListener listener : array )
		{
			listener.propertyValueChanged( name, oldValue, newValue );
		}
	}
	
	public boolean dependsOn( AbstractWsdlModelItem modelItem )
	{
		return false;
	}
	
	public String getTestStepTitle()
	{
		return getTestCase().getTestSuite().getName() + "#" + getTestCase().getName();
	}

	/**
	 * Called after cloning for custom behaviour
	 * 
	 * @param sourceStep step we were cloned from
	 */
	
	public WsdlTestStep clone( WsdlTestCase targetTestCase, String name)
	{
		onSave();
		TestStepConfig newConfig = (TestStepConfig) getConfig().copy();
		newConfig.setName( name );
		return targetTestCase.addTestStep( newConfig );
	}
	
	public void finish( TestRunner testRunner, TestRunContext testRunContext )
	{
	}

	public void prepare( TestRunner testRunner, TestRunContext testRunContext ) throws Exception 
	{
	}

	public Collection<WsdlInterface> getRequiredInterfaces()
	{
		return new ArrayList<WsdlInterface>();
	}
	
	public boolean isDisabled()
	{
		return getConfig().getDisabled();
	}
	
	public void setDisabled( boolean disabled )
	{
		boolean oldDisabled = isDisabled();
		
		if( disabled )
			getConfig().setDisabled( disabled );
		else
			getConfig().unsetDisabled();
		
		notifyPropertyChanged( DISABLED_PROPERTY, oldDisabled, disabled );
	}
}
