/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.tools;

import java.io.OutputStreamWriter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.monitor.TestMonitor;
import com.eviware.soapui.support.UISupport;

public abstract class AbstractSoapUIRunner
{
	private boolean groovyLogInitialized;
	private String projectFile;
	protected static Logger log;

	public AbstractSoapUIRunner( String title )
	{
		log = Logger.getLogger( this.getClass() );
		
		if( title != null )
			System.out.println( title );
		
		SoapUI.initSoapUILog();
		SoapUI.setTestMonitor( new TestMonitor() );
		SoapUI.loadExtLibs();
	}

	public void initGroovyLog()
	{
		if( !groovyLogInitialized )
		{
			Logger logger = Logger.getLogger( "groovy.log" );
			
			ConsoleAppender appender = new ConsoleAppender();
			appender.setWriter( new OutputStreamWriter( System.out ));
			appender.setLayout( new PatternLayout( "%d{ABSOLUTE} %-5p [%c{1}] %m%n") );
			logger.addAppender( appender);
			
			groovyLogInitialized = true;
		}
	}
	
	public void runFromCommandLine( String [] args )
	{
		try
		{
			initFromCommandLine( args, true );
			run();
			System.exit( 0 );
		}
		catch( Throwable e )
		{
			log.error( e.toString() );
			SoapUI.logError( e );
			System.exit( 1 );
		}
	}
	
	public boolean initFromCommandLine( String[] args, boolean printHelp ) throws Exception
	{
		SoapUIOptions options = initCommandLineOptions();
		
		CommandLineParser parser = new PosixParser();
		CommandLine cmd = parser.parse( options, args);
		
		if( options.requiresProject() )
		{
			args = cmd.getArgs();
			
			if( args.length != 1 )
			{
				if( printHelp )
				{
					HelpFormatter formatter = new HelpFormatter();
					formatter.printHelp( options.getRunnerName() + " [options] <soapui-project-file>", options );
				}
	
				System.err.println( "Missing soapUI project file.." );
				return false;
			}
			
			setProjectFile( args[0] );
		}
		
		return processCommandLine( cmd );
	}

	public void enableSwingUI()
	{
		log.info( "Enabling Swing UI" );
		SoapUI.initSoapUILookAndFeel();
		SoapUI.prepareSwingUI();
		UISupport.setMainFrame( null );
	}
	
	protected abstract boolean processCommandLine( CommandLine cmd );

	protected abstract SoapUIOptions initCommandLineOptions();
	
	public abstract void run() throws Exception;

	public String getProjectFile()
	{
		return projectFile;
	}

	/**
	 * Sets the soapUI project file containing the tests to run
	 * 
	 * @param projectFile the soapUI project file containing the tests to run
	 */
	
	public void setProjectFile(String projectFile)
	{
		log.info( "setting projectFile to [" + projectFile + "]" );
		this.projectFile = projectFile;
	}
	
	public static class SoapUIOptions extends Options
	{
		private final String runnerName;

		public SoapUIOptions( String runnerName )
		{
			this.runnerName = runnerName;}

		public String getRunnerName()
		{
			return runnerName;
		}
		
		public boolean requiresProject()
		{
			return true;
		}
	}
}
